<?php

/**
 * Creates content types in code.
 *
 * @code
 * $content_type = CINC::init('ContentType')->set('type', 'my_content_type');
 * @endcode
 */
class CINC_ContentType extends CINC implements CINC_CRUD {

  public $original_config = array();
  public $fields = array();

  function __construct() {

    $this->config_filters['type'] = 'set_type';

    // Set defaults.

    $this->config = array(
      'type' => '',
      'name' => '',
      'base' => 'node_content',
      'description' => '',
      'help' => '',
      'custom' => 1,
      'modified' => 1,
      'locked' => 0,
      'disabled' => 0,
      'is_new' => 1,
      'title_label' => 'Title',
    );

    $this->original_config = $this->config;

    return $this;

  }

  /**
   * Auto-adds name when type is set, and cleans type.
   */
  public function set_type($type) {
    if (empty($this->config['name'])) {
      $this->set('name', CINC::display_name_from_machine_name($type));
    }
    return CINC::clean_machine_name($type);
  }

  /**
   * Saves the content type to the database.
   *
   * @code
   * new CINC_ContentType('my_content_type')
   *   ->save();
   * @endcode
   */
  public function create() {
    $content_type = node_type_set_defaults($this->config);
    node_type_save($content_type);

    return $this;
  }

  /**
   * Loads content type information from the database.
   */
  public function read() {
    $type = node_type_load($this->config['type']);
    $this->config = FALSE;

    if ($type !== FALSE) {
      $this->config = (array) $type;
    }

    return $this;
  }

  /**
   * Loads field information from the database.
   */
  public function read_fields() {
    return $this;
  }

  /**
   * Saves content type to the database and migrates previous if machine name changed.
   */
  public function update() {
    $this->create();

    if ($this->config['type'] != $this->original_config['type']) {
      $original = new CINC_ContentType($this->original_config['type']);
      $original->read()->delete();
    }

    return $this;
  }

  /**
   * Removes content type from the database.
   */
  public function delete() {
    node_type_delete($this->config['type']);

    return $this;
  }

  /**
   * Adds field (instance) to the content type.
   *
   * @param CINC_Field $field_instance
   *
   * @return $this
   */
  public function add_field($field, $weight = NULL) {
    if (!($field instanceof CINC_Field)) {
      throw new Exception('CINC_ContentType::add_field requires a CINC_Field instance as the first parameter');
    }

    // Use the CINC_Field Class method to save the field to this content type.
    $field->create_instance_for('node', $this->config['type'], $weight);
    // Add this field to the fields array property.
    if ($is_saved) {
      array_push($field, $this->fields);
    }

    return $this;
  }

}