<?php

/**
 * Creates fields that can be saved to content types.
 *
 * @code
 * $my_field = CINC::init('Field')->set('field_name', 'my_field');
 * @endcode
 */
class CINC_Field extends CINC implements CINC_CRUD {

  public $instance_config = array();

  function __construct() {

    $this->config_filters['field_name'] = 'set_field_name';

    $this->config = array(
      'field_name' => '',
      'active' => 1,
      'locked' => 0,
      'cardinality' => 1,
      'deleted' => 0,
    );

    $this->instance_config = array(
      'label' => '',
      'required' => 0,
      'description' => '',
      'field_name' => '',
      'deleted' => 0,
      'settings' => array(
        'user_register_form' => FALSE,
      ),
      'widget' => array(
        'active' => 1,
      ),
    );

  }

  /**
   * Auto-adds label when field_name is set, and cleans field_name.
   */
  public function set_field_name($field_name) {
    if (empty($this->instance_config['label'])) {
      $this->set_instance('label', CINC::display_name_from_machine_name($field_name));
    }
    if (empty($this->instance_config['field_name'])) {
      $this->set_instance('field_name', $field_name);
    }
    return CINC::clean_machine_name($field_name);
  }

  /**
   * Sets properties of the field instance.
   *
   * @code
   * CINC::init('Field')
   *   ->set('field_name', 'my_field')
   *   ->set_instance('settings', array('user_register_form' => TRUE);
   * @endcode
   *
   * @param string $property
   *   Property to set.
   * @param string|array $value
   *   Value to set on property.
   * @return $this
   */
  public function set_instance($property, $value) {
    $this->set_on_array_property($property, $value, 'instance_config');

    return $this;
  }

  /**
   * Saves field information to the database.
   */
  public function create() {
    field_create_field($this->config);

    return $this;
  }

  /**
   * Loads field information from the database.
   */
  public function read() {
    $this->config = field_read_field($this->config['field_name'], array('include_inactive' => TRUE));
    return $this;
  }

  /**
   * Loads field information from the database.
   */
  public function read_instance_for($entity_type, $bundle) {
    $this->instance_config = field_read_instance($entity_type, $this->config['field_name'], $bundle);
    return $this;
  }

  /**
   * Saves field information to the database and migrates previous data if machine name changed.
   */
  public function update() {
    field_update_field($this->config);
    return $this;
  }

  /**
   * Removes field information from the database.
   */
  public function delete() {
    field_delete_field($this->config['field_name']);
    return $this;
  }

  public function delete_and_purge() {
    $this->delete();
    field_purge_batch(10);
  }

  /**
   * Saves a field to an entity bundle.
   *
   * @code
   * $field = CINC::init('Field')->set('field_name', 'my_field')->create();
   * $field->create_instance_for('node', 'my_content_type');
   * @endcode
   *
   * @param string $entity_type
   * @param string $bundle
   * @param integer $weight
   */
  public function create_instance_for($entity_type, $bundle, $weight = NULL) {

    $instance = $this->instance_config;
    $instance['entity_type'] = $entity_type;
    $instance['bundle'] = $bundle;

    if ($weight !== NULL) {
      $instance['widget']['weight'] = $weight;
    }

    field_create_instance($instance);

    return $this;
  }

}